# DiscoverPy 
Python wrapper for accessing Polly Discover Data Lake APIs 

## Installation
    
```    
git clone https://bitbucket.org/elucidatainc/discoverpy/src/master/
pip3 install discoverpy/
```

## Contribution
For contributing and adding features refer to [this](docs/contrib.md)


## Usage

```
from discoverpy import Discover
discover = Discover()
```
### Get available repositories
```
discover.get_repositories()
```

Choose a repository from the list of repositories and use it's corresponding it to set the discover object to point to that repository.

### Set a repository for fetching the different endpoints.

- For single cell repositories use `mode='single_cell`.
- For bulk data repositories use `mode='bulk'` (default)

For `geo` repository `repo_id` is 16.
```
discover.set_repo('16')
```
or 
For `sc_data_lake` repository `repo_id` is 17. 
```
discover.set_repo("17", mode="single_cell")
```

### Common methods:- 

1. Get fields available for querying a repository/endpoint.
    ```
    discover.sample_repo.get_all_fields()
    ```

    ```
    ['title', 'geo_accession', 'status', 'submission_date', 'last_update_date', 'type', 'channel_count', 'source_name_ch1', 'organism_ch1', 'characteristics_ch1', 'characteristics_ch1_1', 'treatment_protocol_ch1', 'molecule_ch1', 'extract_protocol_ch1', 'label_ch1', 'label_protocol_ch1', 'taxid_ch1', 'hyb_protocol', 'scan_protocol', 'description', 'data_processing', 'platform_id', 'contact_name', 'contact_email', 'contact_laboratory', 'contact_department', 'contact_institute', 'contact_address', 'contact_city', 'contact_state', 'contact_zip_postal_code', 'contact_country', 'supplementary_file', 'data_row_count', 'relation', 'karyotype_ch1', 'npm1_ch1', 'sample_id', 'tissue', 'cell_type', 'normal_or_diseased', 'treatment', 'kw_column', 'kw_timestamp', 'kw_doc_id']
    ```

    
    ```
    discover.feature_repo.get_all_fields()
    ```
    
    ```
   ['kw_column', 'kw_doc_id', 'kw_index', 'kw_timestamp', 'dataset_source',
   'disease', 'organism', 'platform', 'tissue', 'treatment', 'year']
    ```

2. Get top n examples of entries in the repository/endpoint
    
    ```
    discover.sample_repo.get_top_n_examples(n = 100)
    ```

    ```
    discover.feature_repo.get_top_n_examples(n = 100)
    ```

3. Get datasets which satisfies a condition for a particular field
    
    ```
    discover.dataset_repo.query_dataset_by_field("tissue", "bone", n = 100)
    ```

4. Get datasets which satisfy multiple condition combined using boolean logic

    ```
     discover.dataset_repo.query_dataset_by_field_combination(
         and_fields={"platform":"Microarray", "tissue":"stem cells"},
         not_fields={"description":"gene expression"}, n = 50)
    ```

### Sample repo methods

1. Get samples which satisfy a particular condition for one fields
    
    ```
    discover.sample_repo.query_samples_by_field(self, {"tissue": "blood"}, n = 100):
    ```

2. Get samples which satisfy conditions for multiple fields
    
    ```
    discover.sample_repo.query_samples_by_fields(self, "blood", n = 100):
    ```

3. Get samples which satisfy values for fields and combine them using boolean logic

    ```
    discover.sample_repo.query_samples_by_field_combination(
        and_fields =  {"treatment_protocol_ch1":"mononuclear","source_name_ch1":", 
        not_fields = {"cell_type_ch1":"CD34"}, n = 100)
    ```

    ```
    discover.sample_repo.query_samples_by_field_combination(
        or_fields = {"cell_type_ch1":"CD34","source_name_ch1":"pretreatment"},
        not_fields = {"treatment_protocol_ch1":"de novo"}, n = 300) 
    ```

### Feature repo methods

1. Get value of a feature in multiple datasets
    
    ```
    discover.feature_repo.query_features_by_field({"kw_index":"NRSN1"}, n =100)
    ```

2. Get value of multiple features in all datasets

3. Get value of multiple features in a subset of datasets

## Repository and Package creation

1. Import Repository

    ```
    from discoverpy import Repository
    ```

2. Create a repository object

    ```
    repository = Repository('DEV Test Repo 1', 'DEV Test Repo 1 Description', 'DDD', explorer_enabled=True)
    ```

3. this will create a new repository entry

    ```
    repository.create()
    ```

4. before doing this, make sure you've added newly created repository in your license

    ```
    repository.add_package('dev_test_repo_1_package/data')
    ```
