import requests
import json
from pandas.io.json import json_normalize
import pandas as pd

from .discover import Discover
from .helpers import make_discover_request
from . import env

class AnnotationRepo(Discover):
    def __init__(self, endpoint):
        self.endpoint = endpoint

    def get_annotation_databases(self):
        temp = (self.endpoint).split("_")[0] + "_files"
        self.set_dataset_repo(temp)
        files = self.dataset_repo.query_dataset_by_field(n=1000, field='kw_filetype', value='csv', regexp=True)
        data_lake = (files["_id"].iloc[0].split("@@@")[1]).split("@@")[0]
        database_list = list()
        for i in range (0, files.shape[0]):
            database_list.append(files["_id"].iloc[i].split("@@")[4].split(".")[0])
        return {'database_list': database_list, 'data_lake': data_lake}

    def get_feature_annotation(self, data_lake, _docid, list_of_genes, n=10000):
        chunk_size = 1000
        x = [list_of_genes[i * chunk_size:(i + 1) * chunk_size] for i in range((len(list_of_genes) + chunk_size - 1) // chunk_size )]
        #TODO: move the logic in below two lines to separate functions
        _docid = env.S3_DATALAKE_BUCKET+ "@@@" + data_lake + "@@data@@gene_annotation@@" + _docid + ".csv"
        url = self.host + '/' + self.endpoint + '/_search'
        appended_data = []
        for chunk in x:
            gene_should = []
            for each in chunk:
                gene_match = {
                        "match": {
                        "gene_id": each
                        }
                    }
                gene_should.append(gene_match)
            payload = {
                "size": n,
                "query": {
                    "bool": {
                        "must": [
                            {"match": {"kw_doc_id.keyword": _docid}},
                            {"bool": {"should": gene_should}}
                        ]
                    }
                }
            }
            r = make_discover_request('GET', url, json=payload)
            output = r.json()
            search_df = json_normalize(output["hits"]["hits"])
            appended_data.append(search_df)
        appended_data = pd.concat(appended_data)
        return appended_data
